@@ -345,7 +345,7 @@ class LspGetPortSecurityCommand(cmd.BaseCommand):
         self.result = lsp.port_security
 
 
-class LspGetUpCommand(cmd.BaseCommand):
+class LspGetUpCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, port):
         super(LspGetUpCommand, self).__init__(api)
         self.port = port
@@ -367,7 +367,7 @@ class LspSetEnabledCommand(cmd.BaseCommand):
         lsp.enabled = self.is_enabled
 
 
-class LspGetEnabledCommand(cmd.BaseCommand):
+class LspGetEnabledCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, port):
         super(LspGetEnabledCommand, self).__init__(api)
         self.port = port
@@ -389,7 +389,7 @@ class LspSetTypeCommand(cmd.BaseCommand):
         lsp.type = self.port_type
 
 
-class LspGetTypeCommand(cmd.BaseCommand):
+class LspGetTypeCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, port):
         super(LspGetTypeCommand, self).__init__(api)
         self.port = port
@@ -410,7 +410,7 @@ class LspSetOptionsCommand(cmd.BaseCommand):
         lsp.options = self.options
 
 
-class LspGetOptionsCommand(cmd.BaseCommand):
+class LspGetOptionsCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, port):
         super(LspGetOptionsCommand, self).__init__(api)
         self.port = port
@@ -431,7 +431,7 @@ class LspSetDhcpV4OptionsCommand(cmd.BaseCommand):
         lsp.dhcpv4_options = self.dhcpopt_uuid
 
 
-class LspGetDhcpV4OptionsCommand(cmd.BaseCommand):
+class LspGetDhcpV4OptionsCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, port):
         super(LspGetDhcpV4OptionsCommand, self).__init__(api)
         self.port = port
@@ -468,7 +468,7 @@ class DhcpOptionsDelCommand(cmd.BaseCommand):
         dhcpopt.delete()
 
 
-class DhcpOptionsListCommand(cmd.BaseCommand):
+class DhcpOptionsListCommand(cmd.ReadOnlyCommand):
     def run_idl(self, txn):
         self.result = [rowview.RowView(r) for
                        r in self.api.tables['DHCP_Options'].rows.values()]
@@ -489,7 +489,7 @@ class DhcpOptionsSetOptionsCommand(cmd.BaseCommand):
         dhcpopt.options = self.options
 
 
-class DhcpOptionsGetOptionsCommand(cmd.BaseCommand):
+class DhcpOptionsGetOptionsCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, dhcpopt_uuid):
         super(DhcpOptionsGetOptionsCommand, self).__init__(api)
         self.dhcpopt_uuid = dhcpopt_uuid
@@ -544,7 +544,7 @@ class LrDelCommand(cmd.BaseCommand):
             raise RuntimeError(msg)
 
 
-class LrListCommand(cmd.BaseCommand):
+class LrListCommand(cmd.ReadOnlyCommand):
     def run_idl(self, txn):
         self.result = [rowview.RowView(r) for
                        r in self.api.tables['Logical_Router'].rows.values()]
@@ -610,7 +610,7 @@ class LrpDelCommand(PortDelCommand):
             if_exists)
 
 
-class LrpListCommand(cmd.BaseCommand):
+class LrpListCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, router):
         super(LrpListCommand, self).__init__(api)
         self.router = router
@@ -631,7 +631,7 @@ class LrpSetEnabledCommand(cmd.BaseCommand):
         lrp.enabled = self.is_enabled
 
 
-class LrpGetEnabledCommand(cmd.BaseCommand):
+class LrpGetEnabledCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, port):
         super(LrpGetEnabledCommand, self).__init__(api)
         self.port = port
@@ -712,7 +712,7 @@ class LrRouteDelCommand(cmd.BaseCommand):
             raise RuntimeError(msg)
 
 
-class LrRouteListCommand(cmd.BaseCommand):
+class LrRouteListCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, router):
         super(LrRouteListCommand, self).__init__(api)
         self.router = router
@@ -821,7 +821,7 @@ class LrNatDelCommand(cmd.BaseCommand):
                                        match=self.match_ip)
 
 
-class LrNatListCommand(cmd.BaseCommand):
+class LrNatListCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, router):
         super(LrNatListCommand, self).__init__(api)
         self.router = router
@@ -888,7 +888,7 @@ class LbDelCommand(cmd.BaseCommand):
                 raise
 
 
-class LbListCommand(cmd.BaseCommand):
+class LbListCommand(cmd.ReadOnlyCommand):
     def run_idl(self, txn):
         self.result = [rowview.RowView(r)
                        for r in self.api.tables['Load_Balancer'].rows.values()]
@@ -933,7 +933,7 @@ class LrLbDelCommand(cmd.BaseCommand):
             raise
 
 
-class LrLbListCommand(cmd.BaseCommand):
+class LrLbListCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, router):
         super(LrLbListCommand, self).__init__(api)
         self.router = router
@@ -982,7 +982,7 @@ class LsLbDelCommand(cmd.BaseCommand):
             raise
 
 
-class LsLbListCommand(cmd.BaseCommand):
+class LsLbListCommand(cmd.ReadOnlyCommand):
     def __init__(self, api, switch):
         super(LsLbListCommand, self).__init__(api)
         self.switch = switch
@@ -990,3 +990,64 @@ class LsLbListCommand(cmd.BaseCommand):
     def run_idl(self, txn):
         ls = self.api.lookup('Logical_Switch', self.switch)
         self.result = [rowview.RowView(r) for r in ls.load_balancer]
+
+
+class DnsAddCommand(cmd.AddCommand):
+    table_name = 'DNS'
+
+    def __init__(self, api, **columns):
+        super(DnsAddCommand, self).__init__(api)
+        self.columns = columns
+
+    def run_idl(self, txn):
+        dns = txn.insert(self.api.tables[self.table_name])
+        # Transaction will not be commited if the row is not initialized with
+        # any columns.
+        dns.external_ids = {}
+        self.set_columns(dns, **self.columns)
+        self.result = dns.uuid
+
+
+class DnsDelCommand(cmd.DbDestroyCommand):
+    def __init__(self, api, uuid):
+        super(DnsDelCommand, self).__init__(api, 'DNS', uuid)
+
+
+class DnsGetCommand(cmd.BaseGetRowCommand):
+    table = 'DNS'
+
+
+class DnsListCommand(cmd.ReadOnlyCommand):
+    def run_idl(self, txn):
+        table = self.api.tables['DNS']
+        self.result = [rowview.RowView(r) for r in table.rows.values()]
+
+
+class DnsSetRecordsCommand(cmd.BaseCommand):
+    def __init__(self, api, row_uuid, **records):
+        super(DnsSetRecordsCommand, self).__init__(api)
+        self.row_uuid = row_uuid
+        self.records = records
+
+    def run_idl(self, txn):
+        try:
+            dns = self.api.lookup('DNS', self.row_uuid)
+            dns.records = self.records
+        except idlutils.RowNotFound:
+            msg = "DNS %s does not exist" % self.row_uuid
+            raise RuntimeError(msg)
+
+
+class DnsSetExternalIdsCommand(cmd.BaseCommand):
+    def __init__(self, api, row_uuid, **external_ids):
+        super(DnsSetExternalIdsCommand, self).__init__(api)
+        self.row_uuid = row_uuid
+        self.external_ids = external_ids
+
+    def run_idl(self, txn):
+        try:
+            dns = self.api.lookup('DNS', self.row_uuid)
+            dns.external_ids = self.external_ids
+        except idlutils.RowNotFound:
+            msg = "DNS %s does not exist" % self.row_uuid
+            raise RuntimeError(msg)
